from __future__ import annotations
from geniusweb.actions.PartyId import PartyId
from geniusweb.issuevalue.Bid import Bid
from typing import Dict


class Agreements:
	'''
	There can be multiple agreements from a single negotiation. This object
	contains the agreements. Each party can agree only on 1 bid and there are
	always at least 2 parties agreeing on a bid.
	'''
	def __init__(self, agrees:Dict[PartyId, Bid] =  {} ) :
		'''
		@param agrees a map, with the bid that was agreed on by a party
		'''
		if type(agrees)!=dict:
			raise ValueError("Agreements must be a dict")
		self._agreements = agrees


	def With(self, other:Agreements ) ->Agreements :
		'''
		* @param other {@link Agreements}
		* @return a new Agreement containing this plus other agreements
		* @throws IllegalArgumentException if parties in other are already in this
		*    agreement.
		'''
		newagrees = self._agreements.copy()
		for pid in  other._agreements.keys():
			if pid in newagrees:
				raise ValueError("party " + str(pid) + " already has agreement");
			newagrees[pid]=other._agreements[pid]
		return Agreements(newagrees);

	def getMap(self)-> Dict[PartyId, Bid]: 
		'''
		@return actual agreemenets contained here
		'''
		return self._agreements.copy()


	def __str__(self): 
		return "Agreements" + str(self._agreements)
	
